#ifndef BITTREX_STREAMCODEC_INC
#define BITTREX_STREAMCODEC_INC

#include <string>
#include <stdint.h>
#include "dxapi/schema.h"

#include "dxapi/dxapi.h"
#include "dfp/DecimalNative.hpp"
#include "src/codecs/NativeMessage.h"

#include "Dx_BittrexPackageHeader.h"
#include "Dx_ConnectionStatusChangeMessage.h"
#include "Dx_SecurityStatusMessage.h"


using namespace std;
using namespace DxApi;

namespace dx_BITTREX {

enum MessageTypes {
    Unknown = -1,
    Dx_BittrexPackageHeaderType,
    Dx_ConnectionStatusChangeMessageType,
    Dx_SecurityStatusMessageType,

    Count
};

class Dx_BITTREXStreamDecoder {
private:
    shared_ptr<TickCursor> cursor = nullptr;
    DataReader &reader;

    unsigned knownMessages[MessageTypes::Count];
    std::string guids[MessageTypes::Count];

    InstrumentMessage header;

    dx_BITTREX::deltix::qsrv::hf::plugins::data::bittrex::types::Dx_BittrexPackageHeader dx_bittrexpackageheader;
    dx_BITTREX::deltix::timebase::api::messages::service::Dx_ConnectionStatusChangeMessage dx_connectionstatuschangemessage;
    dx_BITTREX::deltix::timebase::api::messages::status::Dx_SecurityStatusMessage dx_securitystatusmessage;

    NativeMessage *current;

public:
    Dx_BITTREXStreamDecoder(TickStream *stream, shared_ptr<TickCursor> cursor) : reader(cursor->getReader()) {
        this->cursor = cursor;
        this->header.cursor = cursor.get();

        Nullable<string> md = stream->metadata();
        if (!md.has_value())
            throw runtime_error("Stream metadata is empty.");

        vector<Schema::TickDbClassDescriptor> descriptors = Schema::TickDbClassDescriptor::parseDescriptors(md.get(), false);
        for (int i = 0; i < descriptors.size(); ++i) {
            if (!descriptors[i].className.compare("deltix.qsrv.hf.plugins.data.bittrex.types.BittrexPackageHeader"))
                guids[Dx_BittrexPackageHeaderType] = descriptors[i].guid;
            else if (!descriptors[i].className.compare("deltix.timebase.api.messages.service.ConnectionStatusChangeMessage"))
                guids[Dx_ConnectionStatusChangeMessageType] = descriptors[i].guid;
            else if (!descriptors[i].className.compare("deltix.timebase.api.messages.status.SecurityStatusMessage"))
                guids[Dx_SecurityStatusMessageType] = descriptors[i].guid;

        }

        memset(knownMessages, MessageTypes::Unknown, sizeof(knownMessages));
    }

    bool next() {
        if (!cursor->next(&header))
            return false;

        if (knownMessages[header.typeId] == MessageTypes::Unknown) {
            const string * typeName = cursor->getMessageTypeName(header.typeId);
            if (!typeName->compare("deltix.qsrv.hf.plugins.data.bittrex.types.BittrexPackageHeader")) {
                knownMessages[header.typeId] = Dx_BittrexPackageHeaderType;
                //if (guids[Dx_BittrexPackageHeaderType].compare(dx_bittrexpackageheader.getGuid()))
                //    throw runtime_error("Type guid mismatch for type " + dx_bittrexpackageheader.getTypeName());
            } else if (!typeName->compare("deltix.timebase.api.messages.service.ConnectionStatusChangeMessage")) {
                knownMessages[header.typeId] = Dx_ConnectionStatusChangeMessageType;
                //if (guids[Dx_ConnectionStatusChangeMessageType].compare(dx_connectionstatuschangemessage.getGuid()))
                //    throw runtime_error("Type guid mismatch for type " + dx_connectionstatuschangemessage.getTypeName());
            } else if (!typeName->compare("deltix.timebase.api.messages.status.SecurityStatusMessage")) {
                knownMessages[header.typeId] = Dx_SecurityStatusMessageType;
                //if (guids[Dx_SecurityStatusMessageType].compare(dx_securitystatusmessage.getGuid()))
                //    throw runtime_error("Type guid mismatch for type " + dx_securitystatusmessage.getTypeName());
            }

        }

        if (knownMessages[header.typeId] == Dx_BittrexPackageHeaderType) {
            dx_bittrexpackageheader.decode(reader);
            current = &dx_bittrexpackageheader;
        } else if (knownMessages[header.typeId] == Dx_ConnectionStatusChangeMessageType) {
            dx_connectionstatuschangemessage.decode(reader);
            current = &dx_connectionstatuschangemessage;
        } else if (knownMessages[header.typeId] == Dx_SecurityStatusMessageType) {
            dx_securitystatusmessage.decode(reader);
            current = &dx_securitystatusmessage;

        } else {
            throw runtime_error("Unsupported message!");
        }

        *(static_cast<MessageHeader *>(current)) = header;

        return true;
    }

    NativeMessage * getMessage() {
        return current;
    }
};

class Dx_BITTREXStreamEncoder {
private:
    shared_ptr<TickLoader> loader = nullptr;
    DataWriter &writer;

    std::string guids[MessageTypes::Count];

public:
    Dx_BITTREXStreamEncoder(shared_ptr<TickLoader> loader) : writer(loader->getWriter()) {
        this->loader = loader;

        Nullable<string> md = loader->stream()->metadata();
        if (!md.has_value())
            throw runtime_error("Stream metadata is empty.");

        vector<Schema::TickDbClassDescriptor> descriptors = Schema::TickDbClassDescriptor::parseDescriptors(md.get(), false);
        for (int i = 0; i < descriptors.size(); ++i) {
            if (!descriptors[i].className.compare("deltix.qsrv.hf.plugins.data.bittrex.types.BittrexPackageHeader"))
                guids[Dx_BittrexPackageHeaderType] = descriptors[i].guid;
            else if (!descriptors[i].className.compare("deltix.timebase.api.messages.service.ConnectionStatusChangeMessage"))
                guids[Dx_ConnectionStatusChangeMessageType] = descriptors[i].guid;
            else if (!descriptors[i].className.compare("deltix.timebase.api.messages.status.SecurityStatusMessage"))
                guids[Dx_SecurityStatusMessageType] = descriptors[i].guid;

        }

        loader->registerMessageType(Dx_BittrexPackageHeaderType, "deltix.qsrv.hf.plugins.data.bittrex.types.BittrexPackageHeader");
        loader->registerMessageType(Dx_ConnectionStatusChangeMessageType, "deltix.timebase.api.messages.service.ConnectionStatusChangeMessage");
        loader->registerMessageType(Dx_SecurityStatusMessageType, "deltix.timebase.api.messages.status.SecurityStatusMessage");

    }

    void send(NativeMessage &msg) {
        if (msg.typeId < 0 || msg.typeId >= MessageTypes::Count)
            throw runtime_error("Unknown message type: " + msg.getTypeName());

        //if (guids[msg.typeId].compare(msg.getGuid()))
        //    throw runtime_error("Type guid mismatch for type " + msg.getTypeName() +
        //        ". Actual: " + msg.getGuid() + ", Expected: " + guids[msg.typeId]);

        loader->beginMessage(msg.typeId, msg.entityId, msg.timestamp);
        msg.encode(writer);
        loader->send();
    }

};

} // namespace dx_BITTREX

#endif //BITTREX_STREAMCODEC_INC

